/******************************************************************************
 *
 * $Id$
 *
 * Project:  WindNinja Qt GUI
 * Purpose:  Splash screen that shows a pixmap and n messages for m seconds
 *           each.
 * Author:   Kyle Shannon <ksshannon@gmail.com>
 *
 ******************************************************************************
 *
 * THIS SOFTWARE WAS DEVELOPED AT THE ROCKY MOUNTAIN RESEARCH STATION (RMRS)
 * MISSOULA FIRE SCIENCES LABORATORY BY EMPLOYEES OF THE FEDERAL GOVERNMENT
 * IN THE COURSE OF THEIR OFFICIAL DUTIES. PURSUANT TO TITLE 17 SECTION 105
 * OF THE UNITED STATES CODE, THIS SOFTWARE IS NOT SUBJECT TO COPYRIGHT
 * PROTECTION AND IS IN THE PUBLIC DOMAIN. RMRS MISSOULA FIRE SCIENCES
 * LABORATORY ASSUMES NO RESPONSIBILITY WHATSOEVER FOR ITS USE BY OTHER
 * PARTIES,  AND MAKES NO GUARANTEES, EXPRESSED OR IMPLIED, ABOUT ITS QUALITY,
 * RELIABILITY, OR ANY OTHER CHARACTERISTIC.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 *****************************************************************************/

#ifndef SPLASH_H_
#define SPLASH_H_

#include <QTimer>
#include <QSplashScreen>
#include <QStringList>
#include <QMouseEvent>
#include <QPainter>
#include <QDebug>

#define FRAMES_PER_MESSAGE 20

/**
  * \brief Class to display an image and an array of string messages
  *
  * It 'fakes' loading to display images forcefully.  Users can click on the
  * image to bypass
  */
class splashScreen : public QSplashScreen
{
Q_OBJECT

public:
    splashScreen(const QPixmap &pixmap, QStringList list, int time);
    QStringList stringList;
    void display();
    QTimer *messageTimer;
    int messageTime;
    int nMessages;
    int nFrames;
    int i, j;
    float fade_interval;
    int bDone;

    QPixmap map;
    QPixmap orig_map;

    Qt::Alignment alignment;

    QString pad;

signals:
    void done();

protected:
    void mousePressEvent(QMouseEvent *event);

public slots:
    void update();
};

#endif /* SPLASH_H_ */

